#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Feb 12 14:55:37 2024

@author: eemili
"""

import os
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import matplotlib.lines as lines
import matplotlib.dates as mdates
import matplotlib.ticker as ticker
from matplotlib.ticker import MaxNLocator
from datetime import datetime as dt
from datetime import timedelta as td

inidate = dt.fromisoformat('2007-01-01T03')
fstep = td(hours=21)
nsteps = 4380
# nsteps = 740
# dateplt = (dt.strptime('2024042100', '%Y%m%d%H'),dt.strptime('2024042400', '%Y%m%d%H'))
epitype = {'light':1,'moderate':2,'extreme':3}

explist, exptype, expcolor = [], [], []
# explist.append('/esarchive/exp/monarch/a5zv')
# exptype.append('OPER-FCST')
# expcolor.append(['navy','royalblue','lightsteelblue'])

explist.append('/esarchive/scratch/eemili/episodes/dustclim/av_an')
exptype.append('DustRA')
expcolor.append(['skyblue','goldenrod','sienna'])

markers = ['o', 'diamond', 'thin_diamond']
msizes = [1 , 0.6, 0.4, 0.2]

outdir = '/esarchive/scratch/eemili/episodes'

if not os.path.exists(outdir): os.makedirs(outdir)

dates = [inidate+i*td(hours=24) for i in range(nsteps)]

if 'dateplt' not in globals(): dateplt = (dates[0],dates[-1])

fig = plt.figure(figsize=(16,22),dpi=300)
fig.autofmt_xdate()
hours = mdates.HourLocator(interval=6)

axepid = fig.add_subplot(411)
axepid.set_title("Episode intensity")
    
axepid.grid(True)   
axepid.axis(xmin=dateplt[0],xmax=dateplt[1],ymin=0,ymax=4)
axepid.set_xlabel('Time (UTC)')
axepid.yaxis.set_major_locator(MaxNLocator(integer=True))
# axepid.xaxis.set_minor_locator(hours)
# axepid.xaxis.set_minor_formatter(mdates.DateFormatter('%H'))

axplume = fig.add_subplot(412)
axplume.set_title("Plumes area")
axplume.set_ylabel("%s" % ('km2'))
                     
axplume.grid(True)   
axplume.axis(xmin=dateplt[0],xmax=dateplt[1],ymin=0,ymax=3e6)
axplume.set_xlabel('Time (UTC)')
# axplume.xaxis.set_minor_locator(hours)
# axplume.xaxis.set_minor_formatter(mdates.DateFormatter('%H'))

axepiv = fig.add_subplot(413)
axepiv.set_title("Number of daily episodes")
    
axepiv.grid(True)   
axepiv.axis(xmin=dateplt[0],xmax=dateplt[1])
axepiv.set_xlabel('Time (UTC)')
axepiv.yaxis.set_major_locator(MaxNLocator(integer=True))

axepia = fig.add_subplot(414)
axepia.set_title("Anomaly")
    
axepia.grid(True)   
axepia.axis(xmin=dateplt[0],xmax=dateplt[1])
axepia.set_xlabel('Time (UTC)')
axepia.yaxis.set_major_locator(MaxNLocator(integer=True))

expstyles = np.zeros(len(explist),dtype=np.int8)

dailyepi = {exp:np.ma.zeros(len(dates)) for exp in exptype}

for idate,date in enumerate(dates):
    
    for iexp,exp in enumerate(explist):
        
        npfile = os.path.join(explist[iexp],'plots',date.strftime('%Y%m%d'),
                              'episodes_%s_time_series.npz' % (date.strftime('%Y%m%d')))

        if os.path.exists(npfile):
            
            d = np.load(npfile,allow_pickle=True)
            
            # Use different styles for each day of forecast
            leadtime = d['time']-d['time'][0]
            styleidx = np.array([int(lt.total_seconds()/fstep.total_seconds()) for lt in leadtime])
                
            expstyles[iexp] = np.unique(styleidx).size
            
            # axepid.plot(d['time'],d['episodes'],color=expcolor[iexp][0],
            #         ls='-',lw=3,marker=None,label='Forecast',markersize=4.0)
            
            # Consider as episode only events with a persistance of more than 12 hours per day
            if (d['episodes'] > 2).sum() > 3: 
                dailyepi[exptype[iexp]][idate] = 3
            elif (d['episodes'] > 1).sum() > 3: 
                dailyepi[exptype[iexp]][idate] = 2
            elif (d['episodes'] > 0).sum() > 3:
                dailyepi[exptype[iexp]][idate] = 1
            else:
                dailyepi[exptype[iexp]][idate] = 0
            
            axepid.bar(d['time'],d['episodes'],color=expcolor[iexp][0],
                    label='3-hourly')
            
            for ikw,kw in enumerate(epitype):
                axplume.plot(d['time'],d[kw],color=expcolor[iexp][ikw],
                        ls='-',lw=1.5,marker=None,label=kw,markersize=2.0)
            
            # for ist,style in enumerate(np.unique(styleidx)):
                
            #     time = d['time'][styleidx == ist]
            #     episodes = d['episodes'][styleidx == ist]


years = [ dt(year=i,month=1,day=1) for i in range(dates[0].year,dates[-1].year+1)]
yearlyepi = {exp:np.ma.zeros((len(years),len(epitype))) for exp in exptype}
datesyear = np.ma.array([d.year for d in  dates])
for iexp, exp in enumerate(dailyepi):
    axepid.plot(dates,dailyepi[exp],color='black',
               label='Daily',ls='',marker='*',markersize=2.0)

    for ie, epi in enumerate(epitype):
        for iy, year in enumerate(years):
            yearlyepi[exp][iy,ie] = (np.ma.masked_where(datesyear != year.year, dailyepi[exp], copy=True) == epitype[epi]).sum()

        axepiv.plot(years,yearlyepi[exp][:,ie],color=expcolor[iexp][ie],lw=3)
        axepia.plot(years,yearlyepi[exp][:,ie]-np.average(yearlyepi[exp][:,ie]),color=expcolor[iexp][ie],lw=3)

handles = []

# Hardcoded legends
# explegends = [['OPER-FCST D0','OPER-FCST D1'],['FIRST GUESS D0','ANALYSIS D0'],['DA-FCST D0','DA-FCST D1']]

# for iexp,exp in enumerate(explist):
#     for istyle in range(expstyles[iexp]):
#         handles.append(lines.Line2D([0],[0],color=expcolor[iexp][istyle],lw=3,label=exptype[iexp]+'+%i' % (istyle)))
        # handles.append(lines.Line2D([0],[0],color=expcolor[iexp][istyle],lw=3,label=explegends[iexp][istyle]))

axplume.legend(handles=handles)           

prefixname = ''
for exp in explist: prefixname += exp.split('/')[-1]+'_'
plt.savefig(os.path.join(outdir,'%sepisodes_time_series.png' % (prefixname)))
plt.close(fig)