# -*- coding: utf-8 -*-
"""

LETKF plot suite

Plots hourly output data from MONARCH LETKF, compute misfits,
innovation statistics, model fields, analysis increments
and ensemble standard deviation

TODO:
    - [ ] use experiment namelist to set script parameters
    - [ ] generalize instrument type (only VIIRS DOD currently)
"""

import os
import numpy as np
import shutil
from netCDF4 import Dataset
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import obsdb
from plotters import plot2Dmap
from datetime import datetime as dt
from datetime import timedelta as td
from scipy.interpolate import griddata
import copy
import subprocess
from argparse import ArgumentParser, RawDescriptionHelpFormatter

def loopovertime(opt):
    
    firstdate = dt.fromisoformat('%sT00' % (opt.date))
    ndays = int(opt.ndays)
    dates = [firstdate + i*td(days=1) for i in range(ndays)]
    mvar = 'od550du'
    # mvar = 'sconcdu'
    # dastep = td(hours=1)
    dastep = td(hours=3)
    dawindow = td(hours=24)
    # isregional = False
    isregional = True
    enssize = 12
    # MONARCH grid information (used to rotate back observations lats,lons)
    cenlon, cenlat = 20., 35.
    # cenlon is tph0d, cenlat is tlm0d
    # exppath = '/esarchive/exp/monarch/a009_cirrus'
    exppath = '/esarchive/exp/monarch/%s' % (opt.exp)
    mpath = os.path.join(exppath,'regional/3hourly/ensemble-stats')
    pltbounds = {'LatMin':-11,'LatMax':70,'LonMin':-62,'LonMax':102}
    # pltbounds = {'LatMin':15,'LatMax':55,'LonMin':-10,'LonMax':50}
    # pltbounds = {'LatMin':-90,'LatMax':90,'LonMin':-180,'LonMax':180}
    maprange = {'od550du':{'max':1.0,'min':0.},'sconcdu':{'max':1.0e-6,'min':0.}}
    mapdiffrange = {'od550du':{'max':0.5,'min':-0.5},'sconcdu':{'max':0.2e-6,'min':-0.2e-6}}
    mapstdrange = {'od550du':{'max':0.5,'min':0.},'sconcdu':{'max':0.4e-6,'min':0.}}
    maplowdiffrange =  {'od550du':{'max':0.3,'min':-0.3},'sconcdu':{'max':0.1e-6,'min':-0.1e-6}}
    histrange = {'od550du':{'max':1.5,'min':0.}}
    histdiffrange = {'od550du':{'max':1.5,'min':-1.5}}
    histlowdiffrange = {'od550du':{'max':1.,'min':-1.}}
    markersize = 1.0
    
    # Jeronimo CMUG
    # firstdate = dt.fromisoformat('%sT00' % (opt.date))
    # ndays = int(opt.ndays)
    # dates = [firstdate + i*td(days=1) for i in range(ndays)]
    # # date = '2020060100'
    # mvar = 'od550du'
    # # mvar = 'sconcdu'
    # # dastep = td(hours=1)
    # dastep = td(hours=1)
    # dawindow = td(hours=24)
    # isregional = False
    # enssize = 20
    # # MONARCH grid information (used to rotate back observations lats,lons)
    # cenlon, cenlat = 20., 35.
    # # cenlon is tph0d, cenlat is tlm0d
    # exppath = '/esarchive/exp/monarch/a7lg'
    # mpath = os.path.join(exppath,'global/hourly/ensemble-stats')
    # # pltbounds = {'LatMin':-11,'LatMax':70,'LonMin':-62,'LonMax':102}
    # # pltbounds = {'LatMin':15,'LatMax':55,'LonMin':-10,'LonMax':50}
    # pltbounds = {'LatMin':-90,'LatMax':90,'LonMin':-180,'LonMax':180}
    # maprange = {'od550du':{'max':1.0,'min':0.},'sconcdu':{'max':1.0e-6,'min':0.}}
    # mapdiffrange = {'od550du':{'max':0.5,'min':-0.5},'sconcdu':{'max':0.2e-6,'min':-0.2e-6}}
    # mapstdrange = {'od550du':{'max':0.5,'min':0.},'sconcdu':{'max':0.4e-6,'min':0.}}
    # maplowdiffrange =  {'od550du':{'max':0.3,'min':-0.3},'sconcdu':{'max':0.1e-6,'min':-0.1e-6}}
    # histrange = {'od550du':{'max':1.5,'min':0.}}
    # histdiffrange = {'od550du':{'max':1.5,'min':-1.5}}
    # histlowdiffrange = {'od550du':{'max':1.,'min':-1.}}
    # markersize = 1.0
    
    for idate, date in enumerate(dates):
        
        # dpath = os.path.join(exppath)
        dpath = os.path.join(exppath,'original_files/%03i/%s' % (enssize-1,date.strftime('%Y%m%d%H')))
        
        outdir = os.path.join(exppath,'plots',date.strftime('%Y%m%d'))
        # outdir = os.path.join('/esarchive/scratch/eemili/jero-cmug','plots',date.strftime('%Y%m%d'))
        framedir = os.path.join(outdir,'frames')
        gifdir = os.path.join(outdir,'gifs')
        
        if not os.path.exists(outdir): os.makedirs(outdir)
        if not os.path.exists(framedir): os.makedirs(framedir)
        if not os.path.exists(gifdir): os.makedirs(gifdir)
        
        inidate = date
        dasteps = [inidate+i*dastep for i in range(int(dawindow/dastep)+1)]
        
        obsfile = os.path.join(dpath,'dep.txt')
        rejobsfile = os.path.join(dpath,'dep_rej.txt')
        
        yall = obsdb.DefInstrument('VIIRS','VIIRS_AOD',wavelength=[550])
        fgdep = obsdb.DefInstrument('VIIRS','VIIRS_AOD',wavelength=[550])
        nobs, nused, nrej, y_ts, hofxbg_ts, hofxan_ts, fgdep_ts, andep_ts, absinno_ts, fgensstd_ts = [[] for i in range(10)]
        dtslot=int(dastep.total_seconds()/3600)
        nslots=int(dawindow.total_seconds()/dastep.total_seconds())
        # Now loop on output frequency and do some plots
        # for step in dasteps[1:]:
        for step in dasteps[1:]:
            istep = int((step-inidate)/dastep)
            print(step)
            print(istep)
            
            if mvar == 'od550du':
                # Read and plot observations, departures and hofxbg. 
                # Object are reinitialized to avoid cumulating the obs for the moment
                # y = obsdb.DefInstrument('VIIRS','VIIRS_AOD',wavelength=[550])
                # yall = obsdb.DefInstrument('VIIRS','VIIRS_AOD',wavelength=[550])
                # fgdep = obsdb.DefInstrument('VIIRS','VIIRS_AOD',wavelength=[550])
                # Read the obs here from the LETFK output (consider appending the obs, it should be automatically done)
                # but take care of the correct time with this slot shit
                # y.ReadLETKFAscii(obsfile,inidate,islot=istep,dtslot=dtslot,nslots=nslots,rotated=isregional,cenlon=cenlon,cenlat=cenlat)
                fgdep.ReadLETKFAscii(obsfile,inidate,islot=istep,dtslot=dtslot,nslots=nslots,ReadDA={'ObsType':'dep'},rotated=isregional,cenlon=cenlon,cenlat=cenlat)
                fgdep.ReadLETKFAscii(rejobsfile,inidate,islot=istep,dtslot=dtslot,nslots=nslots,ReadDA={'ObsType':'dep'},rotated=isregional,cenlon=cenlon,cenlat=cenlat)
                naccepted = yall.ReadLETKFAscii(obsfile,inidate,islot=istep,dtslot=dtslot,nslots=nslots,rotated=isregional,cenlon=cenlon,cenlat=cenlat)
                nrejected = yall.ReadLETKFAscii(rejobsfile,inidate,islot=istep,dtslot=dtslot,nslots=nslots,rotated=isregional,cenlon=cenlon,cenlat=cenlat)
                # Flag rejected obs
                if nrejected > 0: yall.assim_mask[-nrejected:] = 0
            
                # I can interpolate both hofx_fg and hofx_an. Alternatively the first could be read from file.
                # hofxbg = y - fgdep
                # hofxbg = copy.deepcopy(y)
                # hofxan = copy.deepcopy(y)
                # hofxan = copy.deepcopy(y)
                # fgensstd = copy.deepcopy(y)
            
            # FG ensemble average
            stype = 'av'
            mfile = os.path.join(mpath,'%s_%s' % (mvar,stype),'%s_%s_%s.nc' % (mvar,inidate.strftime('%Y%m%d%H'),stype))
            f = Dataset(mfile, 'r', format="NETCDF4")
            xfg = f[mvar][istep,0,:,:]
            
            if isregional:
                lat_grid = f['lat'][:]
                lon_grid = f['lon'][:]
            else:
                grid = np.meshgrid(f['lon'][:],f['lat'][:])
                lat_grid = grid[1]
                lon_grid = grid[0]
                
            f.close()
            
            # # AN ensemble average
            stype = 'av_an'
            mfile = os.path.join(mpath,'%s_%s' % (mvar,stype),'%s_%s_%s.nc' % (mvar,inidate.strftime('%Y%m%d%H'),stype))
            f = Dataset(mfile, 'r', format="NETCDF4")
            xan = f[mvar][istep,0,:,:]
            f.close()
            
            # stype = 'av_inc'
            # mfile = os.path.join(mpath,'%s_%s' % (mvar,stype),'%s_%s_%s.nc' % (mvar,inidate.strftime('%Y%m%d%H'),stype))
            # f = Dataset(mfile, 'r', format="NETCDF4")
            # xinc = f[mvar][istep,0,:,:]
            # f.close()
            
            # # FG ensemble standard deviation
            # stype = 'std'
            # mfile = os.path.join(mpath,'%s_%s' % (mvar,stype),'%s_%s_%s.nc' % (mvar,inidate.strftime('%Y%m%d%H'),stype))
            # f = Dataset(mfile, 'r', format="NETCDF4")
            # xfg_std = f[mvar][istep,0,:,:]
            # f.close()
            
            # # AN ensemble standard deviation
            # stype = 'std_an'
            # mfile = os.path.join(mpath,'%s_%s' % (mvar,stype),'%s_%s_%s.nc' % (mvar,inidate.strftime('%Y%m%d%H'),stype))
            # f = Dataset(mfile, 'r', format="NETCDF4")
            # xan_std = f[mvar][istep,0,:,:]
            # f.close()
            
            # if not xfg.mask.any():
            # plot2Dmap(xfg,lat_grid,lon_grid,range=maprange[mvar],
            #             title=step.strftime('%Y-%m-%d-%H')+' FG\n Avg:%.2e Min:%.2e Max:%.2e' % (xfg.mean(),xfg.min(),xfg.max()),
            #             plotregion=pltbounds,
            #             filename=os.path.join(framedir,'monarch-%s-%s-fg' % (mvar,step.strftime('%Y%m%d%H'))))
            
            # plot2Dmap(xan,lat_grid,lon_grid,range=maprange[mvar],
            #             title=step.strftime('%Y-%m-%d-%H')+' AN\n Avg:%.2e Min:%.2e Max:%.2e' % (xan.mean(),xan.min(),xan.max()),
            #             plotregion=pltbounds,
            #             filename=os.path.join(framedir,'monarch-%s-%s-an' % (mvar,step.strftime('%Y%m%d%H'))))
            
            # plot2Dmap(xinc,lat_grid,lon_grid,range=mapdiffrange[mvar],
            #             title=step.isoformat()+' INC \n Avg:%.2e Min:%.2e Max:%.2e' % (xinc.mean(),xinc.min(),xinc.max()),
            #             plotregion=pltbounds,
            #             filename=os.path.join(framedir,'monarch-%s-%s-inc' % (mvar,step.strftime('%Y%m%d%H'))),colormap='RdBu_r')
            
            # plot2Dmap(xfg_std,lat_grid,lon_grid,range=mapstdrange[mvar],
            #             title=step.strftime('%Y-%m-%d-%H')+' FG std\n Avg:%.2e Min:%.2e Max:%.2e' % (xfg_std.mean(),xfg_std.min(),xfg_std.max()),
            #             plotregion=pltbounds,
            #             filename=os.path.join(framedir,'monarch-%s-%s-fg-std' % (mvar,step.strftime('%Y%m%d%H'))))
            
            # plot2Dmap(xan_std,lat_grid,lon_grid,range=mapstdrange[mvar],
            #             title=step.strftime('%Y-%m-%d-%H')+' AN std\n Avg:%.2e Min:%.2e Max:%.2e' % (xan_std.mean(),xan_std.min(),xan_std.max()),
            #             plotregion=pltbounds,
            #             filename=os.path.join(framedir,'monarch-%s-%s-an-std' % (mvar,step.strftime('%Y%m%d%H'))))
        
            # -------------------------------------------------------------------------
            # Interpolate analysis data at the observation locations to get the innovation values
            # This is temporary, it should disappear when the LETKF will produce all diags files
            
            # if mvar == 'od550du':
                
                # Interpolate the first guess average at obs locations
                # interpolated_data = griddata(
                #     (lon_grid.ravel(), lat_grid.ravel()),  # Known points
                #     xfg.ravel(),  # Known data
                #     np.array((y.lons[~y.mask], y.lats[~y.mask])).T,  # Points to interpolate to
                #     method='linear'  # You can use 'linear', 'nearest', or other methods
                # )
            
                # hofxbg.aod['550'][~y.mask] = np.ma.masked_invalid(interpolated_data)
                # # Update mask based on possible occurence of NaN values
                # # (this should not happen, it's probably an issue with the interpolator, raise a big flag)
                # hofxbg.UpdateMask()
                # fgdep = y - hofxbg
                # fgdep.UpdateMask()
                
                # Interpolate the analysis average at obs locations
                # interpolated_data = griddata(
                #     (lon_grid.ravel(), lat_grid.ravel()),  # Known points
                #     xan.ravel(),  # Known data
                #     np.array((y.lons[~y.mask], y.lats[~y.mask])).T,  # Points to interpolate to
                #     method='linear'  # You can use 'linear', 'nearest', or other methods
                # )
            
                # hofxan.aod['550'][~y.mask] = np.ma.masked_invalid(interpolated_data)
                # # Update mask based on possible occurence of NaN values
                # # (this should not happen, it's probably an issue with the interpolator, raise a big flag)
                # hofxan.UpdateMask()
                # andep = y - hofxan
                # andep.UpdateMask()
        
                # absinno = andep.abs_sub(fgdep)
                
                # # Interpolate the ensemble standard deviation at obs locations
                # interpolated_data = griddata(
                #     (lon_grid.ravel(), lat_grid.ravel()),  # Known points
                #     xfg_std.ravel(),  # Known data
                #     np.array((y.lons[~y.mask], y.lats[~y.mask])).T,  # Points to interpolate to
                #     method='linear'  # You can use 'linear', 'nearest', or other methods
                # )
                # fgensstd.aod['550'][~y.mask] = np.ma.masked_invalid(interpolated_data)
                # # Update mask based on possible occurence of NaN values
                # # (this should not happen, it's probably an issue with the interpolator, raise a big flag)
                # fgensstd.UpdateMask()
                # -------------------------------------------------------------------------
                
                # hofxbg.PlotObsValues(ObsOnMapOpt={'range':maprange[mvar],'markersize':markersize,
                #                         'colormap':'YlOrBr'},
                #                 PlotRegion=pltbounds,MapOpt={'AddGrid':False},
                #                 outNameSuffix='hofxbg',outdir=framedir,subtitle=step.strftime('%Y-%m-%d-%H'))
                # hofxbg.PlotObsHisto(vmin=histrange[mvar]['min'],vmax=histrange[mvar]['max'],
                #                     outNameSuffix='hofxbg',outdir=framedir,subtitle='hofxbg '+step.strftime('%Y-%m-%d-%H'))
                
                # hofxan.PlotObsValues(ObsOnMapOpt={'range':maprange[mvar],'markersize':markersize,
                #                         'colormap':'YlOrBr'},
                #                 PlotRegion=pltbounds,MapOpt={'AddGrid':False},
                #                 outNameSuffix='hofxan',outdir=framedir,subtitle=step.strftime('%Y-%m-%d-%H'))
                # hofxan.PlotObsHisto(vmin=histrange[mvar]['min'],vmax=histrange[mvar]['max'],
                #                     outNameSuffix='hofxan',outdir=framedir,subtitle='hofxan '+step.strftime('%Y-%m-%d-%H'))
                
                # fgdep.PlotObsValues(ObsOnMapOpt={'range':mapdiffrange[mvar],'markersize':markersize,
                #                         'colormap':'RdBu_r'},
                #                 PlotRegion=pltbounds,MapOpt={'AddGrid':False},
                #                 outNameSuffix='fgdep',outdir=framedir,subtitle=step.strftime('%Y-%m-%d-%H'))
                # fgdep.PlotObsHisto(vmin=histdiffrange[mvar]['min'],vmax=histdiffrange[mvar]['max'],
                #                     outNameSuffix='fgdep',outdir=framedir,subtitle='fgdep '+step.strftime('%Y-%m-%d-%H'))
                
                # andep.PlotObsValues(ObsOnMapOpt={'range':mapdiffrange[mvar],'markersize':markersize,
                #                     'colormap':'RdBu_r'},
                #             PlotRegion=pltbounds,MapOpt={'AddGrid':False},
                #             outNameSuffix='andep',outdir=framedir,subtitle=step.strftime('%Y-%m-%d-%H'))
                # andep.PlotObsHisto(vmin=histdiffrange[mvar]['min'],vmax=histdiffrange[mvar]['max'],
                #                     outNameSuffix='andep',outdir=framedir,subtitle='andep '+step.strftime('%Y-%m-%d-%H'))
                
                # absinno.PlotObsValues(ObsOnMapOpt={'range':maplowdiffrange[mvar],'markersize':markersize,
                #                 'colormap':'RdBu_r'},
                #         PlotRegion=pltbounds,MapOpt={'AddGrid':False},
                #         outNameSuffix='andep-fgdep',outdir=framedir,subtitle=step.strftime('%Y-%m-%d-%H'))
                # absinno.PlotObsHisto(vmin=histlowdiffrange[mvar]['min'],vmax=histlowdiffrange[mvar]['max'],
                #                 outNameSuffix='andep-fgdep',outdir=framedir,subtitle='|andep|-|fgdep| '+step.strftime('%Y-%m-%d-%H'))
                
                # yall.PlotMaskValues(MaskOnMapOpt={'range':maprange[mvar],'markersize':markersize},
                #                             PlotRegion=pltbounds,MapOpt={'AddGrid':False},
                #                             outdir=framedir,subtitle=step.strftime('%Y-%m-%d-%H'))
                
                # Compute stats and save them for a time series plot (not perfect solution since the obs
                # plots should be done by the class methods, but I haven't yet developed strategy to handle
                # multiple obs objects within the obs class)
                # nobs.append(yall.nobs)
                # nused.append(naccepted)
                # nrej.append(nrejected)
                # y_ts.append(y.Stats())
                # hofxbg_ts.append(hofxbg.Stats())
                # hofxan_ts.append(hofxan.Stats())
                # fgdep_ts.append(fgdep.Stats())
                # andep_ts.append(andep.Stats())
                # absinno_ts.append(absinno.Stats())
                # fgensstd_ts.append(fgensstd.Stats())
            # This can undo selections if needed
            # y.ResetSel()
        
                    
        # Here we can plot obs for all day if needed
        # yall.PlotObsValues(ObsOnMapOpt={'range':maprange[mvar],'markersize':markersize,
        #                             # 'colormap':'viridis'},
        #                             'colormap':'YlOrBr'},
        #                            PlotRegion=pltbounds,MapOpt={'AddGrid':False},
        #                            outNameSuffix='yall',outdir=outdir)
        # yall.PlotMaskValues(MaskOnMapOpt={'range':maprange[mvar],'markersize':markersize},
        #                            PlotRegion=pltbounds,MapOpt={'AddGrid':False},
        #                            outNameSuffix='assim_mask',outdir=outdir)

if __name__ == "__main__":
    
    parser = ArgumentParser(
      formatter_class=RawDescriptionHelpFormatter,
      description=('''\n\
      Plots results from operational DA experiments.
      E.g. ./postprocda.py -n 1 -d "2024-07-15"
      '''))
    parser.add_argument(
      "-n", "--ndays", dest="ndays",
      help="Number of days to process")
    parser.add_argument(
      "-d", "--date", dest="date",
      help="First date to process")
    parser.add_argument(
      "-e", "--exp", dest="exp",
      help="Name of the experiment")

    parser.set_defaults(ndays=1,date='2024-07-31',exp='a7pb')
    opt = parser.parse_args()
    
    loopovertime(opt)